package aexp.selectlivefolder;

import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import android.content.res.Resources;
import android.content.Intent.ShortcutIconResource;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.Bundle;
import android.provider.LiveFolders;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;

public class SelectLiveFolder extends ListActivity
{
    private static final String LOG_TAG = "SelectLiveFolder";

    private static final int REQUEST_CREATE_INDIVIDUAL_LIVE_FOLDER = 100;

    private static final String KEY_PKG = "pkg";
    private static final String KEY_ACTIVITY = "activity";
    private static final String KEY_URISTRING = "uriString";
    private static final String KEY_DESCRIPTION = "description";
    private static final String KEY_BASEINTENT = "baseIntent";


    ArrayList<HashMap<String,String>> list = 
                    new ArrayList<HashMap<String,String>>();

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        fillLiveFolderActivities( list );
        SimpleAdapter termAdapter = new SimpleAdapter( 
				this, 
				list,
				R.layout.live_folder_row,
				new String[] { KEY_PKG,
                                KEY_ACTIVITY,
                                KEY_URISTRING,
                                KEY_DESCRIPTION,
                                KEY_BASEINTENT },
				new int[] { R.id.pkg, 
                            R.id.activity,
                            R.id.uri,
                            R.id.description,
                            R.id.baseIntent  }  );
        setListAdapter( termAdapter );
        createLiveFolders( list );
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == RESULT_OK ) {
            if( requestCode >= REQUEST_CREATE_INDIVIDUAL_LIVE_FOLDER ) {
                int itemIdx = 
                        requestCode - 
                        REQUEST_CREATE_INDIVIDUAL_LIVE_FOLDER;
                refreshLiveFolder( list, itemIdx, data );
            } 
        }
    }

    @Override
    protected void onListItemClick (ListView l, View v, int position, long id) {
        Log.d( LOG_TAG, "onListItemClick: "+position );
        HashMap<String,String> item = null;
        synchronized( list ) {
            if( position < list.size() )
                item = list.get( position );
        }
        if( item != null ) {
            String uriString = item.get( KEY_URISTRING );
            if( uriString != null ) {
                Uri uri = Uri.parse( uriString );
                Intent intent = new Intent();
                intent.setData( uri );
                intent.setClassName( 
                    "aexp.selectlivefolder", 
                    "aexp.selectlivefolder.DisplayLiveFolder" );
                startActivity( intent );
            }
        }
    }

    private void fillLiveFolderActivities( 
                    ArrayList<HashMap<String,String>> items ) {
        PackageManager packageManager = getPackageManager();
        Intent baseIntent = new Intent(LiveFolders.ACTION_CREATE_LIVE_FOLDER);
        List<ResolveInfo> list = packageManager.queryIntentActivities(baseIntent,
                0  );
        for( int i = 0 ; i < list.size() ; ++i ) {
            ResolveInfo info = list.get( i );
            ActivityInfo ainfo = info.activityInfo;
            if( ainfo != null ) {
                HashMap<String,String> item = new HashMap<String,String>();
                item.put( KEY_PKG, ainfo.packageName );
                item.put( KEY_ACTIVITY, ainfo.name );
                items.add( item );
            }
        }
    }

    private void createLiveFolders( 
                    ArrayList<HashMap<String,String>> items ) {
        for( int i = 0 ; i < items.size() ; ++i ) {
            HashMap<String,String> item = items.get( i );
            String pkg = item.get( KEY_PKG );
            String activity = item.get( KEY_ACTIVITY );
            Intent intent = new Intent( LiveFolders.ACTION_CREATE_LIVE_FOLDER );
            intent.setClassName( pkg, activity );
            startActivityForResult(intent, 
                REQUEST_CREATE_INDIVIDUAL_LIVE_FOLDER+i );
        }
    }

    private void refreshLiveFolder( 
            ArrayList<HashMap<String,String>> items,
            int itemIdx,
            Intent data ) {
        synchronized( items ) {
            HashMap<String,String> item = items.get( itemIdx );
            Uri uri = data.getData();
            String uriString = uri.toString();
            String description = 
                    data.getStringExtra( 
                        LiveFolders.EXTRA_LIVE_FOLDER_NAME );
            Bundle extras = data.getExtras();
            Intent baseIntent = (Intent)extras.get( 
                            LiveFolders.EXTRA_LIVE_FOLDER_BASE_INTENT );
            item.put( KEY_URISTRING, uriString );
            item.put( KEY_DESCRIPTION, description );
            if( baseIntent != null )
                item.put( KEY_BASEINTENT, baseIntent.toString() );
            else
                item.put( KEY_BASEINTENT, "<null>" );
            Log.d( LOG_TAG, "uri: "+uriString );
            Log.d( LOG_TAG, "description: "+description );
            Log.d( LOG_TAG, "base intent: "+baseIntent );
        }
    }


}